#include "StdAfx.h"
#include "math.h"
#include "ImageOp.h"


#define AVERAGE(a,b,c) ( ((a)+(b)+(c))/3 )
#define BLACK 0
#define WHITE 255

inline void pixel2ARGB(int pixel, int &alpha, int &red, int &green, int &blue) {
	alpha = (pixel & 0xff000000) >> 24;
	red = (pixel   & 0x00ff0000) >> 16;
	green = (pixel & 0x0000ff00) >> 8;
	blue = (pixel  & 0x000000ff) >> 0;
}

inline int ARGB2Pixel(int alpha, int r, int g, int b) {
	if ( r > 255 ) r = 255; if ( r < 0 ) r = 0; r = (r << 16); 
	if ( g > 255 ) g = 255; if ( g < 0 ) g = 0; g = (g << 8 ); 
	if ( b > 255 ) b = 255; if ( b < 0 ) b = 0;
	return ( ((alpha << 24 )&0xff000000)  | r | g| b );
}

void ImageOp::GrayScale(int width, int height, int *argb) {
	int a, r, g, b;
	for ( int i=0; i < width*height; i++) {
		pixel2ARGB(argb[i], a,r,g,b);
		r = AVERAGE(r,g,b);
		argb[i] = ARGB2Pixel(a,r,r,r);
	}
}

void ImageOp::Negate(int width, int height, int *argb) {
	int a, r, g, b;
	for ( int i=0; i < width*height; i++) {
		pixel2ARGB(argb[i], a,r,g,b);
		r = 255-r;
		g = 255-g;
		b = 255-b;
		argb[i] = ARGB2Pixel(a,r,g,b);
	}
}

void ImageOp::Threshold(int width, int height, int *argb, int level) {
	int a, r, g, b;
	for ( int i=0; i < width*height; i++) {
		pixel2ARGB(argb[i], a,r,g,b);
		r = AVERAGE(r,g,b);
		r = (r<level)?0:255;
		argb[i] = ARGB2Pixel(a,r,r,r);
	}
}
