// ImgProcDoc.cpp : implementation of the CImgProcDoc class
//

#include "stdafx.h"
#include "ImgProc.h"
#include "ImgProcDoc.h"

#include "ImageOp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CImgProcDoc

IMPLEMENT_DYNCREATE(CImgProcDoc, CDocument)

BEGIN_MESSAGE_MAP(CImgProcDoc, CDocument)
	//{{AFX_MSG_MAP(CImgProcDoc)
	ON_COMMAND(ID_IMAGE_GRAY, OnImageGray)
	ON_COMMAND(ID_IMAGE_NEGATE, OnImageNegate)
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
	ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_REDO, OnUpdateEditRedo)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


///////////////////////////////////////////////////////////////////////
// Below are two macros that are called before and
// afyter any Image processing operation to setup
// undo - redo as well as convert to Bitmap <-> int array
////////////////////////////////////////////////////////////////////////
#define PREOP()		Bitmap *m_Bitmap = GetBitmap();\
					if ( m_Bitmap == NULL ) return;\
					int x = m_Bitmap->GetWidth();\
					CWaitCursor waitCursor();\
					int y = m_Bitmap->GetHeight();\
					int *pixels = NULL;\
					pixels = Bitmap2PixelData(m_Bitmap);\
					if ( pixels == NULL) {\
						AfxMessageBox("Insufficient Memory. Operation Aborted");\
						return ; \
					}\
					undo.push(current);\
					current = NULL;\
					redo.Empty();

#define POSTOP()	current = new URItem(m_Bitmap);\
					delete []pixels;\
					this->UpdateAllViews(NULL);

/////////////////////////////////////////////////////////////////////////////

// CImgProcDoc construction/destruction
CImgProcDoc::CImgProcDoc()
{
	// TODO: add one-time construction code here
	current = NULL;
}

CImgProcDoc::~CImgProcDoc()
{
}

BOOL CImgProcDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)
	ClearDoc();

	return TRUE;
}


// Returns the current bitmap to display.
Bitmap *CImgProcDoc::GetBitmap() {
	if ( current != NULL ) {
		return current->getBitmap();
	}
	return NULL;
}

/////////////////////////////////////////////////////////////////////////////
// CImgProcDoc serialization

void CImgProcDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

void CImgProcDoc::ClearDoc() {
	if ( current != NULL ) {
		delete current;
		current = NULL;
	}
	while ( ! undo.isEmpty() ) {
		URItem *it = undo.pop();
		delete it;
	}

	while ( ! redo.isEmpty() ) {
		URItem *it = redo.pop();
		delete it;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CImgProcDoc diagnostics

#ifdef _DEBUG
void CImgProcDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CImgProcDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CImgProcDoc commands

BOOL CImgProcDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if (!CDocument::OnOpenDocument(lpszPathName))
		return FALSE;

	Bitmap *bitmap;

	ClearDoc(); // SDI Apps resue the doc

	CString filename(lpszPathName);

	BSTR bstrFilename = filename.AllocSysString();
	bitmap = Bitmap::FromFile(bstrFilename);
	::SysFreeString(bstrFilename);

	if ( bitmap == NULL ) return FALSE;

	if ( bitmap->GetHeight() <=0 || bitmap->GetWidth() <=0 ) {
		// invalid bitmap
		AfxMessageBox("Failed to Read File");
		return FALSE;
	}

	current = new URItem(bitmap);
	return TRUE;
}

void CImgProcDoc::OnImageGray() 
{
	PREOP();
	
	ImageOp::GrayScale(x,y,pixels);
	m_Bitmap = PixelData2BitMap(x, y, pixels);

	POSTOP();
	
}

void CImgProcDoc::OnImageNegate() 
{
	PREOP();
	
	ImageOp::Negate(x,y,pixels);
	m_Bitmap = PixelData2BitMap(x, y, pixels);

	POSTOP();

}


int *CImgProcDoc::Bitmap2PixelData(Bitmap *b) {

	int width = b->GetWidth();
	int height = b->GetHeight();

	if ( width *height <= 0 ) 
		return NULL;

	int *pixelData_ARGB = new int[width*height];

	if ( pixelData_ARGB == NULL )
		return NULL;

    BitmapData bmData;
	Rect rect(0, 0, width, height);

	b->LockBits(rect, 
        ImageLockModeRead, 
		PixelFormat32bppARGB,
		&bmData); 

    int stride = bmData.Stride; 
    BYTE *p = (BYTE *)((void *)bmData.Scan0);

	int nOffset = stride - width*4; // bytes to skip at end of each row

	for(int y=0; y < height;y++) {
		for(int x=0; x < width; ++x ) {
			// GDI lies about RGB - internally its BGR
			pixelData_ARGB[y*width+x] =  ARGB2Pixel(p[3], p[2], p[1], p[0]);
			p += 4;
		}
		p += nOffset;
	}
	b->UnlockBits(&bmData);

	return (pixelData_ARGB);
}

Bitmap *CImgProcDoc::PixelData2BitMap(int width, int height, int *pixData_ARGB) {
	Bitmap bit(width, height, PixelFormat32bppARGB);
	Bitmap *b = bit.Clone(0, 0, bit.GetWidth(), bit.GetHeight(), PixelFormat32bppARGB);

    BitmapData bmData;
	Rect rect(0, 0, b->GetWidth(), b->GetHeight());
	b->LockBits(rect, 
        ImageLockModeRead | ImageLockModeWrite, 
		PixelFormat32bppARGB,
		&bmData); 

    int stride = bmData.Stride; 
    BYTE *p = (BYTE *)((void *)bmData.Scan0);
	int nOffset = stride - width*4; // bytes to skip at end of each row

	int pixel;

	for(int y=0; y < height;y++) {
		for(int x=0; x < width; ++x ) {
			// GDI lies about RGB - internally its BGR
			pixel = pixData_ARGB[y*width+x];
			p[3] = (BYTE) ((pixel >> 24) & 0xff); // alpha
			p[2] = (BYTE) ((pixel >> 16) & 0xff); // pixel red
			p[1] = (BYTE) ((pixel >> 8 ) & 0xff); // pixel green
			p[0] = (BYTE) ((pixel      ) & 0xff); // pixel blue
			p += 4;
		}
		p += nOffset;
	}
	b->UnlockBits(&bmData);

	return (b);
}

void CImgProcDoc::OnEditUndo() 
{
	redo.push(current);
	current = undo.pop();
	this->UpdateAllViews(NULL);
}

void CImgProcDoc::OnUpdateEditUndo(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!undo.isEmpty());	
}

void CImgProcDoc::OnEditRedo() 
{
	undo.push(current);
	current = redo.pop();
	this->UpdateAllViews(NULL);
}

void CImgProcDoc::OnUpdateEditRedo(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!redo.isEmpty());	
}
